SystemOrganization addCategory: #'MenuRegistration-example'!
SystemOrganization addCategory: #'MenuRegistration-Core'!

Object subclass: #MenuRegistration
	instanceVariableNames: 'parentName itemList order spec owner isGroup ordering'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MenuRegistration-Core'!

!MenuRegistration commentStamp: 'AlainPlantec 2/16/2010 23:17' prior: 0!
A MenuRegistration stores the declaration of a menu item. It is mainly an handler for a PluggableMenuItemSpec. 

A menu item is declared withing a particular method tagged with a pragma. This kind-of method takes a builder as argument and its evaluation results in the building of a MenuRegistration sub-tree which is stored in the builder.   As an example:
MenuRegistrationExample class>>myOwnCoolToolRegistrationOn: aBuilder 
	<worldMenuExample> 
	(aBuilder item: #MyOwnCoolTool)
		label: 'My own cool tool';
		target: Workspace; 
		selector: #openContents: ;
		arguments: #('yep, my own cool tool can be opened from the world menu !! :)')
	
Evaluating this method results in the creation of a MenuRegistration which name is #MyOwnCoolTool. Thus, the resulting sub-tree is only made of a root node. After it has be built, this root node is recorded in the builder. To experiment this, just evaluate the following code:
-------------
| builder |
builder := PragmaMenuBuilder new.
MenuRegistrationExample myOwnCoolToolRegistrationOn: builder.
builder explore
-------------

Using pragma allows the menu builder to dynamically discover which are the methods to evaluate in order to build a menu. Thus, a resulting menu is built by evaluating a set of methods which share the same pragma keyword.
In the following example, all method having <worldMenuExample> are evaluated for the building of the resulting menu:
---------------
(PragmaMenuBuilder pragmaKeyword: 'worldMenuExample' model: nil) menu popUpInWorld
---------------

Within a method, three kind of declarations can be used: (1) item by item (2) an item with a sub-menu and (3) a group.

1 -  item by item menu registration declaration:
This kind of declaring is for the setting of one menu item and only one within a method.

1.1 -  A simple menu item with an action
In the following example, a menu item with the name #'Browser' and a action which 
consists in sending #openClassBrowser to StandardToolSet is declared:

AClassSomewhere class>>openBrowserOn: aBuilder
	<myMenuTest>
	(aBuilder item: #'Browser') 
		target: StandardToolSet; 
		selector: #openClassBrowser.

A simple action without any argument can also be set with a block:
	(aBuilder item: #'Browser') action: [StandardToolSet openClassBrowser]

You can also indicate a balloon help string and a particular icon:
	(aBuilder item: #'Browser') 
		action: [StandardToolSet openClassBrowser];
		help: 'Open a system browser';
		icon: MenuIcons smallSystemBrowserIcon

If the action needs one or several arguments, you can also give it/them as follow:
	(aBuilder item: #'Save and quit') 
		target: SmalltalkImage current; 
		selector: #snapshot:andQuit:.
		arguments: #(true true)
		
By default, the item label is set with the item name but it can be explicitly given as follow:
	(aBuilder item: #'Browser')
		label: 'System browser'; 
		target: StandardToolSet; 
		selector: #openClassBrowser.

1.2  -  Placing the menu item in a  menu
The resulting menu item of previous example will be placed at the root of the menu.
In order to declare another place for it, you have to explicitly set its parent name.
As an example, consider the following item which declares a simple entry with no action.
Such item is typically used as a root for a sub-menu tree:

AClassSomewhere class>>openToolsOn: aBuilder
	<myMenuTest>
	(aBuilder item: #'Tools')

Now, a sub-menu item for #Tools can be declared separately, within another method
by using the #parent: message:

AnotherClassSomewhere class>>myToolsOn: aBuilder
	<myMenuTest>
	(aBuilder item: #'CoolTool')
		label: 'Cool tool';
		parent: #Tools;

Note that the argument of #parent: must be the name of another item. 
If it is not the case, then the parent name indication is simply ignored.

1.3  -  Item ordering

If no ordering setting is indicated, items ordering is unpredicable (it depends on method retrieving order). If one want an item to appear at a certain position, it is possible to set it by sending #order: to a MenuRegistration. The #order: message takes a float as argument. 
As an example, see  the two following declarations, in the resulting menu, Wozy is placed before 'Wozy configuration'

AnotherClassSomewhere class>>myWozySystemOpenOn: aBuilder
	<myMenuTest>
	(aBuilder item: #'Wozy')
		parent: #CoolTool;
		order: 1.0
		
AnotherClassSomewhere class>>myWozySystemConfigOn: aBuilder
	<myMenuTest>
	(aBuilder item: #'Wozy configuration')
		parent: #CoolTool;
		order: 2.0

2) Item with a submenu
The one menu item - one declaring method way can be ugly. When a set a menu items are known to be put all-together, it is possible to declare the sub-tree in one method. The following example show such a sub-tree with the #Tools item at root and four sub-items declared in a single method. Note a menu target declared for the root is shared by all sub-items. In that case, it also remains possible for a sub-item to declare its own target.

AClassSomewhere class>>openToolsOn: aBuilder
	<myMenuTest>
	(aBuilder item: #'Tools')
		target:  StandardToolSet; "The target is shared by all children"
		with: [ "My sub-menu are given here"
			(aBuilder item: #'System browser') selector: #openClassBrowser.
			(aBuilder item: #Workspace) selector: #openWorkspace.
			(aBuilder item: #'Test Runner') selector: #openTestRunner.
			(aBuilder item: #'Monticello Browser') selector: #openMonticelloBrowser]
		
3) group of menu items
When you want some items to be shown always grouped together, you can use a group. Its declaring is like an item with a submenu except that you are using the message #group: instead of #item:. The consequence is that only the children are shown in the menu. Of course, #label and #icon: are ignored for a group. Here is an example:

AClassSomewhere class>>mostUsedToolsOn: aBuilder
	<myMenuTest>
	(aBuilder group: #MostUsedTools) "My name can be also used as parent name"
		withSeparatorAfter; "A separator will be added after my last child"
		order: 0; "the entire group will be placed at the top"
		target:  StandardToolSet; "The target is shared by all children"
		with: [
			(aBuilder item: #'System browser') selector: #openClassBrowser.
			(aBuilder item: #Workspace) selector: #openWorkspace.
			(aBuilder item: #'Test Runner') selector: #openTestRunner.
			(aBuilder item: #'Monticello Browser') selector: #openMonticelloBrowser]

---------------------------
		
Instance Variables
	isGroup:		<Boolean>
	itemList:		<SortedCollection>
	order:		<Number>
	owner:		<MenuRegistration>
	parentName:		<Symbol>
	spec:		<PluggableMenuItemSpec>

isGroup
	- if true, then this item is ignored and flatten

itemList
	- all my MenuRegistration (my sub-menus or my elements if i'm a group)

order
	- my order in the owner list

owner
	- my owner

parentName
	- the declared parent name which serve as basis for PragmaBenuBuilder>>#arrangeRegistrations

spec
	- my PluggableMenuItemSpec
!

!MenuRegistration class methodsFor: 'instance creation' stamp: 'AlainPlantec 2/16/2010 12:20'!
owner: aMenuRegistration
	^ self new owner: aMenuRegistration! !

!MenuRegistration methodsFor: 'spec accessing' stamp: 'AlainPlantec 2/16/2010 09:57'!
action: aMessageSendOrABlock
	"set the action of the menu"
	
	aMessageSendOrABlock isBlock 
		ifTrue: [
			self target: aMessageSendOrABlock.
			self selector: #value]
		ifFalse: [
			self action: aMessageSendOrABlock]
! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/16/2010 16:45'!
addItem: anItem 
	"Add a MenuRegistration"
	self ensureItemList add: anItem! !

!MenuRegistration methodsFor: 'spec accessing' stamp: 'AlainPlantec 2/13/2010 12:59'!
arguments: anArray
	"set the arguments for an item with an action"
	self spec action 
		ifNil: [self spec action: (MessageSend receiver: self itemReceiver selector: nil arguments: anArray)]
		ifNotNil: [self spec action arguments: anArray]! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/16/2010 19:37'!
assignOrderWithBlock: aBlock 
	"Compute recursively MenuRegistration order"
	self itemList 
		ifNotNil: [self itemList: (aBlock value: self itemList).
				self itemList do: [:rded | rded assignOrderWithBlock: aBlock]]! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/16/2010 16:48'!
buildMenuSpec: aMenu 
	"recursiveley build the menu spec aMenu passed as argument"
	self itemList
		ifNotNil: [:l | 
			| m |
			m := isGroup 
				ifFalse: [ToolBuilder default pluggableMenuSpec withModel: nil]
				ifTrue: [aMenu].
			self itemList do: [:i | i buildMenuSpec: m].
			isGroup ifFalse: [self spec subMenu: m]].
	isGroup
		ifTrue: [self spec separator ifTrue: [self itemList ifNotEmpty: [self itemList last spec separator: true]]] 
		ifFalse: [aMenu items add: self spec]! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/13/2010 12:52'!
builder
	^ owner builder! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/16/2010 16:49'!
collectMisplacedItemsIn: aCollection
	"Select recursively all MenuRegistration with a parentName 
	different from its owner name. 	This is possible since a parent name 
	can be given in order to force a menu item placement. 
	If my parentName is not the same as my owner name then it means 
	that I'm not in the right place and that my placement has to be resolved"
	
	self parentName
		ifNotNil: [owner
				ifNil: [aCollection add: self]
				ifNotNil: [owner name = self parentName
						ifFalse: [aCollection add: self]]].
	self itemList ifNotNil: [:l |
		l do: [:item | item collectMisplacedItemsIn: aCollection]]! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/16/2010 16:45'!
ensureItemList
	"Return the list of MenuRegistration instances"
	^ itemList ifNil: [itemList := OrderedCollection new]! !

!MenuRegistration methodsFor: 'sub item creating' stamp: 'AlainPlantec 2/16/2010 11:42'!
group: aSymbol
	^ (self item: aSymbol) isGroup: true! !

!MenuRegistration methodsFor: 'spec accessing' stamp: 'AlainPlantec 2/12/2010 16:04'!
help: anHelpText
	"set the help text"
	self spec help: anHelpText! !

!MenuRegistration methodsFor: 'spec accessing' stamp: 'AlainPlantec 2/12/2010 22:23'!
icon: aForm
	"set the icon that is shown in the menu"
	self spec icon: aForm! !

!MenuRegistration methodsFor: 'initialize-release' stamp: 'AlainPlantec 2/11/2010 16:35'!
initialize
	super initialize.
	isGroup := false! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/16/2010 11:42'!
isGroup: aBoolean
	isGroup := aBoolean! !

!MenuRegistration methodsFor: 'sub item creating' stamp: 'AlainPlantec 2/16/2010 13:08'!
item: aSymbol
	"set my name and my label with aSymbol (the label and the name as the same by default) and register myself as a child of my owner"
	^ (self builder newSubItem) name: aSymbol; label: aSymbol asString
! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/16/2010 16:45'!
itemList
	"Return my children"
	^ itemList 
! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/15/2010 11:17'!
itemList: aCollection
	"Set the list of MenuRegistration"
	 itemList := aCollection! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/16/2010 16:49'!
itemNamed: aName
	"Return the MenuRegistration named aName or nil if none is found"
	self name = aName ifTrue: [^ self].
	^ self itemList ifNotNil: [:l | l detect: [:item | (item itemNamed: aName) notNil] ifNone: []]
! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/13/2010 12:58'!
itemReceiver
	^ self target ifNil: [owner itemReceiver]! !

!MenuRegistration methodsFor: 'spec accessing' stamp: 'AlainPlantec 2/12/2010 16:06'!
label 
	"return my label"
	^ self spec label! !

!MenuRegistration methodsFor: 'spec accessing' stamp: 'AlainPlantec 2/12/2010 16:06'!
label: aLabel
	"set the label that is shown in the menu"
	self spec label: aLabel! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/13/2010 12:54'!
model
	^ self builder model! !

!MenuRegistration methodsFor: 'spec accessing' stamp: 'AlainPlantec 2/12/2010 16:07'!
name
	"returns my name which is used as a key (should unique over all the menu items"
	^ self spec name! !

!MenuRegistration methodsFor: 'spec accessing' stamp: 'AlainPlantec 2/12/2010 16:07'!
name: aSymbol
	"set my name (should be unique over all menu items"
	self spec name: aSymbol! !

!MenuRegistration methodsFor: 'sub item creating' stamp: 'AlainPlantec 2/16/2010 13:43'!
newSubItem
	| reg |
	reg := self class owner: self.
	self addItem: reg.
	^ reg! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/11/2010 08:15'!
order
	^ order! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/15/2010 11:05'!
order: aNumber 
	"Set the value of order"

	order := aNumber! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/11/2010 08:15'!
owner
	^ owner! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/11/2010 08:15'!
owner: anItem
	owner := anItem! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/10/2010 21:03'!
parent: aSymbol
	self parentName: aSymbol! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/10/2010 13:54'!
parentName
	^ parentName! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/10/2010 13:54'!
parentName: aSymbol
	parentName := aSymbol! !

!MenuRegistration methodsFor: 'initialize-release' stamp: 'AlainPlantec 2/15/2010 11:21'!
release
	itemList := nil.
	super release
	! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/15/2010 11:05'!
removeItem: anItem
	"Remove a MenuRegistration"
	self itemList remove: anItem ! !

!MenuRegistration methodsFor: 'initialize-release' stamp: 'AlainPlantec 2/18/2010 11:53'!
reset
	itemList 
		ifNotNil: [
			itemList do: [:i | i reset].
			itemList := nil].
	spec := nil
! !

!MenuRegistration methodsFor: 'spec accessing' stamp: 'AlainPlantec 2/13/2010 12:58'!
selector: aSymbol
	"set the selector for item with an action"
	self spec action 
		ifNil: [self spec action: (MessageSend receiver: self itemReceiver selector: aSymbol)]
		ifNotNil: [self spec action selector: aSymbol]! !

!MenuRegistration methodsFor: 'accessing' stamp: 'AlainPlantec 2/16/2010 16:51'!
sort: aSortBlock
	"Sort the tree recursively"
	self itemList
		ifNotNil: [
		self itemList: (self itemList asSortedCollection: aSortBlock).
		self itemList do: [:rded | rded sort: aSortBlock] ]! !

!MenuRegistration methodsFor: 'spec accessing' stamp: 'AlainPlantec 2/12/2010 16:08'!
spec
	"return my PluggableMenuItemSpec"
	^ spec ifNil: [spec := PluggableMenuItemSpec new]! !

!MenuRegistration methodsFor: 'spec accessing' stamp: 'AlainPlantec 2/13/2010 12:59'!
target
	^ self spec action ifNotNil: [:action | action receiver]
! !

!MenuRegistration methodsFor: 'spec accessing' stamp: 'AlainPlantec 2/13/2010 13:28'!
target: anObject
	"set the target for the action"
	self spec action 
		ifNil: [self spec action: (MessageSend receiver: (anObject ifNil: [owner itemReceiver]) selector: nil)]
		ifNotNil: [self spec action receiver: anObject]! !

!MenuRegistration methodsFor: 'sub item creating' stamp: 'AlainPlantec 2/16/2010 11:47'!
with: aBlock
	self builder currentRoot: self while: aBlock! !

!MenuRegistration methodsFor: 'spec accessing' stamp: 'AlainPlantec 2/12/2010 16:09'!
withSeparatorAfter
	"add a separator line after me"
	self spec separator: true! !

MenuRegistration subclass: #PragmaMenuBuilder
	instanceVariableNames: 'pragmaKeyword model pragmaCollector currentRoot'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MenuRegistration-Core'!

!PragmaMenuBuilder commentStamp: 'AlainPlantec 2/16/2010 17:55' prior: 0!
For the impatients, see MenuRegistrationExample class methods and try it with:
---------------
((PragmaMenuBuilder pragmaKeyword: MenuRegistrationExample pragmaKeyword model: nil) menuEntitled: 'World') popUpInWorld
---------------

PragmaMenuBuilder is for the dynamic building of menus based on pragmas. A PragmaMenuBuilder instance is the root of a tree of MenuRegistration instances.
The basic principle is that each menu sub-tree is specified by a method which is tagged by a specific pragma. Such methods are dynamically retrieved and then evaluated with a MenuRegistration passed as argument (see #retrieveRegistrations). The result is a tree of MenuRegistration which roots are stored in my itemList inst. var.
After the tree of MenuRegistration has been built, it is re-organized (re-organization is based on the parent declaration) and is re-ordered (based on the MenuRegistration order indications). Then the tree of MenuRegistration can serve as input for the building of a PluggableMenuSpec. The PluggableMenuSpec is itself used in order to build a MenuMorph with the help of the current ToolBuilder. (see MenuRegistration comment for more informations about how to specify menu entries).

The tree of MenuRegistration is built by #buildTree in three steps (1) the  collecting of the MenuRegistration instances (2) the re-organization and (3) the sorting:

1) The first step consists in evaluating all pragma methods by passing a builder (a PragmaMenuBuilder instance) as argument. Each pragma method invocation build a sub-tree which root is added to the builder itemList collection. (see #collectRegistrations).
As an example, this first step could produce a tree as follow (stored in a PragmaMenuBuilder itemList inst var) :
				#Tools						#'Other tool', parentName: #Tools
				/	\										|
	(#Worspace)	(#browser)						(#'Test runner' )

2) The second step consists is re-organizing the tree. A MenuRegistration can be declared with a particular parent name (by sending #parent: to it with a symbol as argument). If the parentName of a MenuRegistration X is the name of another MenuRegistration Z, then it means that X must be placed as a child of Z. This is the goal of this re-arrangement step which moves badly placed nodes at their good place. (see #arrangeRegistrations).
With previous example, the second step produces:
						#Tools								
				/		|			\
	(#Worspace)	(#browser)		#'Other tool' , parentName: #Tools
											|
									(#'Test runner')

2) The third step consists in sorting the tree according to the order inst. var. value of each MenuRegistration. This is done in two passes: the first pass tries to assign as much order inst. var. as possible (If an item is given with a specific order, then, previous and following items order can be automatically computed - see #assignOrderWithBlock: and #orderAssignBlock). The second pass consists in a smple sort according to a sort block given by #itemSortBlock.


Instance Variables
	model:		<Object>
	pragmaCollector:		<PragmaCollection>
	pragmaKeyword:		<String>
	currentRoot: 			<MenuRegistration>

model
	- Serves as the default target for the menu. Note that a default target can also be declared at menu item level

pragmaKeyword
	- The pragma keyword used for the declaring of my menu items

pragmaCollector
	- The PragmaCollector associated with this builder. When a method declared with the same pragma as my pragmaKeyword is updated/added/removed my menu items are recomputed so that the resulting menu is always in sync with currently declared items.
	
currentRoot
	- the current MenuRegistration in which new items are to be added

!

!PragmaMenuBuilder class methodsFor: 'private' stamp: 'AlainPlantec 2/16/2010 19:35'!
itemSortBlock
	"The block which is used to sort a menu tree"
	^ [:a :b | 
		((a order notNil and: [b order notNil]) and: [a order ~= b order])
			ifTrue: [a order < b order]
			ifFalse: [((a order isNil and: [b order isNil]) or: [a order = b order])
				ifTrue: [true]
				ifFalse: [a order ifNil: [false] ifNotNil: [true]]]]! !

!PragmaMenuBuilder class methodsFor: 'private' stamp: 'AlainPlantec 2/12/2010 14:41'!
orderAssignBlock
	"The block which is used to set MenuRegistration tree node order inst var (which is used to sort the tree)"
	^[:list | 
		(list detect: [:n | n order notNil] ifNone:[]) "No order has been set - do not touch anything, the list order is ok"
			ifNil: [list]
			ifNotNil: [ :firstWithOrder | | idx order | 
				"A menu registration with an order indication hase been found - then compute order of previous and next nodes"
				idx := list indexOf: firstWithOrder.
				order := firstWithOrder order.
				idx > 1
					ifTrue: [(idx - 1 to: 1) 
						do: [:pos | 
							(list at: pos) order: order - 1. 
							order := order - 1]].
				order := firstWithOrder order.
				(idx + 1 to: list size) 
					do: [:pos | 
						(list at: pos) order 
							ifNil:  [(list at: pos) order: order + 1] 
							ifNotNil: [order := (list at: pos) order].
						order := order + 1].
						list]]! !

!PragmaMenuBuilder class methodsFor: 'instance creation' stamp: 'AlainPlantec 2/14/2010 10:12'!
pragmaKeyword: aPragmaKeyword model: aModel
	"Build a builder using aPragmaKeyword as the pragma keyword and aModel a the model of the resulting builder"
	^ self new pragmaKeyword: aPragmaKeyword;
		model: aModel;
		buildTree! !

!PragmaMenuBuilder methodsFor: 'registrations handling' stamp: 'AlainPlantec 2/12/2010 14:34'!
allMisplacedItems
	| misplaced |
	self collectMisplacedItemsIn: (misplaced := OrderedCollection new).
	^ misplaced
! !

!PragmaMenuBuilder methodsFor: 'registrations handling' stamp: 'AlainPlantec 2/18/2010 11:46'!
arrangeRegistrations.
	self allMisplacedItems do: [:item |
		(self itemNamed: item parentName)
			ifNotNil: [:newOwner | item owner removeItem: item.
				item owner: newOwner.
				newOwner addItem: item]].
! !

!PragmaMenuBuilder methodsFor: 'menu building' stamp: 'AlainPlantec 2/14/2010 13:58'!
buildTree
	"Retrieve all menu registrations with the help of a PragmaCollector then, 
	reorganise the tree and sort it  - see class comment for more informations"
	itemList
		ifNil: [
			itemList := OrderedCollection new.
			self collectRegistrations.
			self arrangeRegistrations.
			self sortRegistrations]! !

!PragmaMenuBuilder methodsFor: 'accessing' stamp: 'AlainPlantec 2/13/2010 12:52'!
builder
	^ self! !

!PragmaMenuBuilder methodsFor: 'registrations handling' stamp: 'AlainPlantec 2/16/2010 12:21'!
collectRegistrations
	"Retrieve all pragma methods and evaluate them by passing the 
	MenuRegistration class as argument. The result is a list of trees
	stored in my itemList inst var"
	self pragmaCollector
		do: [:prg | self
				currentRoot: self
				while: [prg methodClass theNonMetaClass perform: prg selector with: (MenuRegistration owner: self)]].
! !

!PragmaMenuBuilder methodsFor: 'menu building' stamp: 'AlainPlantec 2/16/2010 16:36'!
currentRoot: anItem while: aBlock
	| old |
	old := currentRoot.
	currentRoot := anItem.
	[aBlock value] ensure: [currentRoot := old]! !

!PragmaMenuBuilder methodsFor: 'initialize-release' stamp: 'AlainPlantec 2/16/2010 16:43'!
initialize
	super initialize.
	isGroup := true.
	currentRoot := self.
	self pragmaCollector whenChangedSend: #reset to: self
	! !

!PragmaMenuBuilder methodsFor: 'accessing' stamp: 'AlainPlantec 2/13/2010 12:57'!
itemReceiver
	^ model! !

!PragmaMenuBuilder methodsFor: 'public menu building' stamp: 'AlainPlantec 2/14/2010 11:05'!
menu
	"returns a MenuMorph build by Toolbuilder from my menuSpec"
	^ ToolBuilder build: self menuSpec! !

!PragmaMenuBuilder methodsFor: 'public menu building' stamp: 'AlainPlantec 2/14/2010 14:02'!
menuAt: aName
	"returns a MenuMorph build by Toolbuilder from my menuSpec"
	^ ToolBuilder build: (self menuSpecAt: aName)! !

!PragmaMenuBuilder methodsFor: 'public menu building' stamp: 'AlainPlantec 2/12/2010 14:18'!
menuEntitled: aTitle 
	"returns a MenuMorph build by Toolbuilder from my menuSpec"
	^ ToolBuilder
		build: (self menuSpec label: aTitle)! !

!PragmaMenuBuilder methodsFor: 'public menu building' stamp: 'AlainPlantec 2/14/2010 14:09'!
menuSpec
	"returns a PluggableMenuSpec build from my contents"
	^ self menuSpecAt: nil! !

!PragmaMenuBuilder methodsFor: 'public menu building' stamp: 'AlainPlantec 2/15/2010 11:26'!
menuSpecAt: aName
	"returns a PluggableMenuSpec build from my contents starting at 
	the inner MenuRegistration named aName or from here if aName is nil"
	| root |
	self buildTree.
	root := ToolBuilder default pluggableMenuSpec withModel: nil.
	(aName ifNil: [self] ifNotNil: [self itemNamed: aName]) 
		ifNotNil: [:top | top buildMenuSpec: root].
	^ root! !

!PragmaMenuBuilder methodsFor: 'accessing' stamp: 'AlainPlantec 2/13/2010 12:40'!
model
	^ model! !

!PragmaMenuBuilder methodsFor: 'accessing' stamp: 'AlainPlantec 2/13/2010 12:40'!
model: anObject
	model := anObject! !

!PragmaMenuBuilder methodsFor: 'menu building' stamp: 'AlainPlantec 2/16/2010 16:58'!
newSubItem
	| reg |
	reg := MenuRegistration owner: currentRoot.
	currentRoot addItem: reg.
	^ reg! !

!PragmaMenuBuilder methodsFor: 'registrations handling' stamp: 'AlainPlantec 2/16/2010 17:59'!
pragmaCollector
	"Return an up-to-date pragmaCollector which contains all pragmas which keyword is self pragmaKeyword"
	^ pragmaCollector 
		ifNil: [pragmaCollector:= (PragmaCollector filter: [:prg | prg keyword = self pragmaKeyword and: [prg selector numArgs = 1]]).
			self pragmaKeyword ifNotNil: [pragmaCollector reset].
			pragmaCollector]! !

!PragmaMenuBuilder methodsFor: 'accessing' stamp: 'AlainPlantec 2/14/2010 10:18'!
pragmaKeyword
	"Returns the pragma keyword used to select pragmas (see #pragmaCollector)"
	^  pragmaKeyword ! !

!PragmaMenuBuilder methodsFor: 'accessing' stamp: 'AlainPlantec 2/14/2010 10:16'!
pragmaKeyword: aString
	"Set the pragma keyword used to select pragmas (see #pragmaCollector)"
	pragmaKeyword := aString.
	self pragmaCollector reset.! !

!PragmaMenuBuilder methodsFor: 'initialize-release' stamp: 'AlainPlantec 2/15/2010 11:21'!
release
	self pragmaCollector unsubscribe: self.
	pragmaCollector := nil.
	model := nil.
	super release
	! !

!PragmaMenuBuilder methodsFor: 'registrations handling' stamp: 'AlainPlantec 2/12/2010 23:21'!
sortRegistrations
	"Try to update MenuRegistration order inst. var and the sort the trees"
	self assignOrderWithBlock: self class orderAssignBlock.
	self sort: self class itemSortBlock! !

Object subclass: #MenuRegistrationExample
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MenuRegistration-example'!

!MenuRegistrationExample commentStamp: 'AlainPlantec 2/17/2010 01:10' prior: 0!
I'm here to show you how to use the menu registration. See class side methods and try it with:
----------------
(PragmaMenuBuilder pragmaKeyword: 'worldMenuExample' model: nil) menu popUpInWorld
----------------
!

!MenuRegistrationExample class methodsFor: 'menu declaring' stamp: 'AlainPlantec 2/16/2010 22:45'!
mostUsedToolsOn: aBuilder 
	<worldMenuExample>
	(aBuilder group: #MostUsedTools)
		withSeparatorAfter;
		order: 0;
		target: StandardToolSet;
		with: [
			(aBuilder item: #'System browser') selector: #openClassBrowser; icon: Browser taskbarIcon.
			(aBuilder item: #Workspace) selector: #openWorkspace; icon: Workspace taskbarIcon.
			(aBuilder item: #'Test Runner') selector: #openTestRunner; icon: TestRunner taskbarIcon.
			(aBuilder item: #'Monticello Browser') selector: #openMonticelloBrowser; icon: MCWorkingCopyBrowser taskbarIcon]! !

!MenuRegistrationExample class methodsFor: 'menu declaring' stamp: 'AlainPlantec 2/17/2010 01:09'!
myOwnCoolMenuRegistrationOn: aBuilder 
	<worldMenuExample>
	(aBuilder item: #MyOwnCoolMenu)
		parent: #Tools; "place me as a child of the #Tools node (declared by #toolsOn:)"
		label: 'The coolest tool here';
		with: [
			(aBuilder item: #MyOwnCoolTool)
				label: 'My own cool tool also here';
				target: Workspace;
				selector: #openContents: ;
				arguments: #('yep, my own cool tool can be also opened from a world sub-menu !! :)')]! !

!MenuRegistrationExample class methodsFor: 'menu declaring' stamp: 'AlainPlantec 2/17/2010 01:09'!
myOwnCoolToolRegistrationOn: aBuilder 
	<worldMenuExample> 
	(aBuilder item: #MyOwnCoolTool)
		parent: #Tools;
		label: 'My own cool tool';
		target: Workspace; 
		selector: #openContents: ;
		arguments: #('yep, my own cool tool can be opened from the world menu !! :)')
! !

!MenuRegistrationExample class methodsFor: 'menu declaring' stamp: 'AlainPlantec 2/12/2010 15:02'!
pragmaKeyword
	^ 'worldMenuExample'! !

!MenuRegistrationExample class methodsFor: 'menu declaring' stamp: 'AlainPlantec 2/17/2010 00:36'!
quitItemsOn: aBuilder 
	<worldMenuExample> 
	(aBuilder group: #QuitPharo)
		order: 9999; 
		with: [{
				{'Save'. {SmalltalkImage current. #saveSession}. 'save the current version of the image on disk'}.
				{'Save as...'. {self. #saveAs}. 'save the current version of the image on disk under a new name.'}.
				{'Save and quit'. {self. #saveAndQuit}. 'save the current image on disk, and quit out of Pharo.'}.
	 			{'Quit'. {self. #quitSession}. 'quit out of Pharo.'}
			} do: [:triplet |
					(aBuilder item: triplet first asSymbol) 
						target: triplet second first;
				 		selector: triplet second second;
						help: (triplet size > 2 ifTrue: [triplet third] ifFalse: [nil])]]! !

!MenuRegistrationExample class methodsFor: 'menu actions' stamp: 'AlainPlantec 2/11/2010 19:43'!
quitSession
	SmalltalkImage current
		snapshot: (self confirm: 'Save changes before quitting?' translated orCancel: [ ^ self ])
		andQuit: true! !

!MenuRegistrationExample class methodsFor: 'menu actions' stamp: 'AlainPlantec 2/11/2010 19:43'!
saveAndQuit
	SmalltalkImage current snapshot: true andQuit: true! !

!MenuRegistrationExample class methodsFor: 'menu actions' stamp: 'AlainPlantec 2/11/2010 19:42'!
saveAs
	| name index |
	name := FileDirectory baseNameFor: (FileDirectory default localNameFor: SmalltalkImage current imageName).
	index := name lastIndexOf: FileDirectory extensionDelimiter ifAbsent: [ nil ].
	(index notNil and: [ (name copyFrom: index + 1 to: name size) isAllDigits ])
		ifTrue: [ name := name copyFrom: 1 to: index - 1 ].
	name := FileDirectory default nextNameFor: name extension: FileDirectory imageSuffix.
	name := UIManager default
		request: 'Please enter the name for the new image:'
		initialAnswer: name.
	name isEmptyOrNil
		ifTrue: [ ^ self ].
	SmalltalkImage current saveAs: name! !

!MenuRegistrationExample class methodsFor: 'menu declaring' stamp: 'AlainPlantec 2/17/2010 01:12'!
toolsOn: aBuilder 
	<worldMenuExample>
	(aBuilder item: #Tools)
		order: 1.0;
		target: StandardToolSet;
		icon: MenuIcons smallDoItIcon;
		withSeparatorAfter;
		with: [(aBuilder item: #Transcript) selector: #openTranscript; icon: TranscriptStream  taskbarIcon. 
			(aBuilder item: #'File Browser') selector: #openFileList; icon: FileList taskbarIcon.
			(aBuilder item: #'Process Browser') selector: #openProcessBrowser; icon: ProcessBrowser taskbarIcon;
				 withSeparatorAfter]! !
